@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2015 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2017 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -26,17 +26,8 @@
 #include	"sndfile.h"
 #include	"sfendian.h"
 #include	"common.h"
-#include	"wav_w64.h"
+#include	"wavlike.h"
 
-/* These required here because we write the header in this file. */
-
-#define RIFF_MARKER	(MAKE_MARKER ('R', 'I', 'F', 'F'))
-#define WAVE_MARKER	(MAKE_MARKER ('W', 'A', 'V', 'E'))
-#define fmt_MARKER	(MAKE_MARKER ('f', 'm', 't', ' '))
-#define fact_MARKER	(MAKE_MARKER ('f', 'a', 'c', 't'))
-#define data_MARKER	(MAKE_MARKER ('d', 'a', 't', 'a'))
-
-#define WAVE_FORMAT_MS_ADPCM	0x0002
 
 typedef struct
 {	int				channels, blocksize, samplesperblock, blocks, dataremaining ;
@@ -61,11 +52,11 @@ static int AdaptationTable [] =
    appear in the actual WAVE file.  They should be read in
    in case a sound program added extras to the list. */
 
-static int AdaptCoeff1 [MSADPCM_ADAPT_COEFF_COUNT] =
+static int AdaptCoeff1 [WAVLIKE_MSADPCM_ADAPT_COEFF_COUNT] =
 {	256, 512, 0, 192, 240, 460, 392
 } ;
 
-static int AdaptCoeff2 [MSADPCM_ADAPT_COEFF_COUNT] =
+static int AdaptCoeff2 [WAVLIKE_MSADPCM_ADAPT_COEFF_COUNT] =
 {	0, -256, 0, 64, 0, -208, -232
 } ;
 
@@ -124,7 +115,7 @@ static	void	choose_predictor (unsigned int channels, short *data, int *bpred, in
 */
 
 int
-wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
+wavlike_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 {	MSADPCM_PRIVATE	*pms ;
 	unsigned int	pmssize ;
 	int				count ;
@@ -137,8 +128,14 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	if (psf->file.mode == SFM_WRITE)
 		samplesperblock = 2 + 2 * (blockalign - 7 * psf->sf.channels) / psf->sf.channels ;
 
-	if (blockalign < 7 * psf->sf.channels)
-	{	psf_log_printf (psf, "*** Error blockalign (%d) should be > %d.\n", blockalign, 7 * psf->sf.channels) ;
+	/* There's 7 samples per channel in the preamble of each block */
+	if (samplesperblock < 7 * psf->sf.channels)
+	{	psf_log_printf (psf, "*** Error samplesperblock (%d) should be >= %d.\n", samplesperblock, 7 * psf->sf.channels) ;
+		return SFE_INTERNAL ;
+		} ;
+
+	if (2 * blockalign < samplesperblock * psf->sf.channels)
+	{	psf_log_printf (psf, "*** Error blockalign (%d) should be >= %d.\n", blockalign, samplesperblock * psf->sf.channels / 2) ;
 		return SFE_INTERNAL ;
 		} ;
 
@@ -200,15 +197,15 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	psf->seek = msadpcm_seek ;
 
 	return 0 ;
-} /* wav_w64_msadpcm_init */
+} /* wavlike_msadpcm_init */
 
 
 static inline short
 msadpcm_get_bpred (SF_PRIVATE *psf, MSADPCM_PRIVATE *pms, unsigned char value)
-{	if (value >= MSADPCM_ADAPT_COEFF_COUNT)
+{	if (value >= WAVLIKE_MSADPCM_ADAPT_COEFF_COUNT)
 	{	if (pms->sync_error == 0)
 		{	pms->sync_error = 1 ;
-			psf_log_printf (psf, "MS ADPCM synchronisation error (%u should be < %u).\n", value, MSADPCM_ADAPT_COEFF_COUNT) ;
+			psf_log_printf (psf, "MS ADPCM synchronisation error (%u should be < %u).\n", value, WAVLIKE_MSADPCM_ADAPT_COEFF_COUNT) ;
 			} ;
 		return 0 ;
 		} ;
@@ -233,7 +230,7 @@ msadpcm_decode_block	(SF_PRIVATE *psf, MSADPCM_PRIVATE *pms)
 		return 1 ;
 		} ;
 
-	if ((k = psf_fread (pms->block, 1, pms->blocksize, psf)) != pms->blocksize)
+	if ((k = (int) psf_fread (pms->block, 1, pms->blocksize, psf)) != pms->blocksize)
 	{	psf_log_printf (psf, "*** Warning : short read (%d != %d).\n", k, pms->blocksize) ;
 		if (k <= 0)
 			return 1 ;
@@ -358,7 +355,7 @@ msadpcm_read_s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	while (len > 0)
 	{	readcount = (len > 0x10000000) ? 0x10000000 : (int) len ;
 
-		if ((count = msadpcm_read_block (psf, pms, ptr, readcount)) <= 0)
+		if ((count = (int) msadpcm_read_block (psf, pms, ptr, readcount)) <= 0)
 			return -1 ;
 
 		total += count ;
@@ -385,9 +382,9 @@ msadpcm_read_i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	sptr = ubuf.sbuf ;
 	bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : len ;
+	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
 
-		if ((count = msadpcm_read_block (psf, pms, sptr, readcount)) <= 0)
+		if ((count = (int) msadpcm_read_block (psf, pms, sptr, readcount)) <= 0)
 			return -1 ;
 
 		for (k = 0 ; k < readcount ; k++)
@@ -417,9 +414,9 @@ msadpcm_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sptr = ubuf.sbuf ;
 	bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : len ;
+	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
 
-		if ((count = msadpcm_read_block (psf, pms, sptr, readcount)) <= 0)
+		if ((count = (int) msadpcm_read_block (psf, pms, sptr, readcount)) <= 0)
 			return -1 ;
 
 		for (k = 0 ; k < readcount ; k++)
@@ -449,9 +446,9 @@ msadpcm_read_d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sptr = ubuf.sbuf ;
 	bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : len ;
+	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
 
-		if ((count = msadpcm_read_block (psf, pms, sptr, readcount)) <= 0)
+		if ((count = (int) msadpcm_read_block (psf, pms, sptr, readcount)) <= 0)
 			return -1 ;
 
 		for (k = 0 ; k < readcount ; k++)
@@ -515,12 +512,12 @@ msadpcm_seek	(SF_PRIVATE *psf, int mode, sf_count_t offset)
 */
 
 void
-msadpcm_write_adapt_coeffs	(SF_PRIVATE *psf)
+wavlike_msadpcm_write_adapt_coeffs	(SF_PRIVATE *psf)
 {	int k ;
 
-	for (k = 0 ; k < MSADPCM_ADAPT_COEFF_COUNT ; k++)
-		psf_binheader_writef (psf, "22", AdaptCoeff1 [k], AdaptCoeff2 [k]) ;
-} /* msadpcm_write_adapt_coeffs */
+	for (k = 0 ; k < WAVLIKE_MSADPCM_ADAPT_COEFF_COUNT ; k++)
+		psf_binheader_writef (psf, "22", BHW2 (AdaptCoeff1 [k]), BHW2 (AdaptCoeff2 [k])) ;
+} /* wavlike_msadpcm_write_adapt_coeffs */
 
 /*==========================================================================================
 */
@@ -529,7 +526,8 @@ static int
 msadpcm_encode_block	(SF_PRIVATE *psf, MSADPCM_PRIVATE *pms)
 {	unsigned int	blockindx ;
 	unsigned char	byte ;
-	int				chan, k, predict, bpred [2], idelta [2], errordelta, newsamp ;
+	int				chan, k, predict, bpred [2] = { 0 }, idelta [2] = { 0 },
+					errordelta, newsamp ;
 
 	choose_predictor (pms->channels, pms->samples, bpred, idelta) ;
 
@@ -635,7 +633,7 @@ msadpcm_encode_block	(SF_PRIVATE *psf, MSADPCM_PRIVATE *pms)
 
 	/* Write the block to disk. */
 
-	if ((k = psf_fwrite (pms->block, 1, pms->blocksize, psf)) != pms->blocksize)
+	if ((k = (int) psf_fwrite (pms->block, 1, pms->blocksize, psf)) != pms->blocksize)
 		psf_log_printf (psf, "*** Warning : short write (%d != %d).\n", k, pms->blocksize) ;
 
 	memset (pms->samples, 0, pms->samplesperblock * sizeof (short)) ;
@@ -681,7 +679,7 @@ msadpcm_write_s	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 	while (len > 0)
 	{	writecount = (len > 0x10000000) ? 0x10000000 : (int) len ;
 
-		count = msadpcm_write_block (psf, pms, ptr, writecount) ;
+		count = (int) msadpcm_write_block (psf, pms, ptr, writecount) ;
 
 		total += count ;
 		len -= count ;
@@ -707,10 +705,10 @@ msadpcm_write_i	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 	sptr = ubuf.sbuf ;
 	bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : len ;
+	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
 		for (k = 0 ; k < writecount ; k++)
 			sptr [k] = ptr [total + k] >> 16 ;
-		count = msadpcm_write_block (psf, pms, sptr, writecount) ;
+		count = (int) msadpcm_write_block (psf, pms, sptr, writecount) ;
 		total += count ;
 		len -= writecount ;
 		if (count != writecount)
@@ -737,10 +735,10 @@ msadpcm_write_f	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 	sptr = ubuf.sbuf ;
 	bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : len ;
+	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
 		for (k = 0 ; k < writecount ; k++)
-			sptr [k] = lrintf (normfact * ptr [total + k]) ;
-		count = msadpcm_write_block (psf, pms, sptr, writecount) ;
+			sptr [k] = psf_lrintf (normfact * ptr [total + k]) ;
+		count = (int) msadpcm_write_block (psf, pms, sptr, writecount) ;
 		total += count ;
 		len -= writecount ;
 		if (count != writecount)
@@ -767,10 +765,10 @@ msadpcm_write_d	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 	sptr = ubuf.sbuf ;
 	bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : len ;
+	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
 		for (k = 0 ; k < writecount ; k++)
-			sptr [k] = lrint (normfact * ptr [total + k]) ;
-		count = msadpcm_write_block (psf, pms, sptr, writecount) ;
+			sptr [k] = psf_lrint (normfact * ptr [total + k]) ;
+		count = (int) msadpcm_write_block (psf, pms, sptr, writecount) ;
 		total += count ;
 		len -= writecount ;
 		if (count != writecount)
