@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2003-2014 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2003-2017 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -92,7 +92,7 @@ xi_open	(SF_PRIVATE *psf)
 
 		/* Set up default instrument and software name. */
 		memcpy (pxi->filename, "Default Name            ", sizeof (pxi->filename)) ;
-		memcpy (pxi->software, PACKAGE "-" VERSION "               ", sizeof (pxi->software)) ;
+		memcpy (pxi->software, PACKAGE_NAME "-" PACKAGE_VERSION "               ", sizeof (pxi->software)) ;
 
 		memset (pxi->sample_name, 0, sizeof (pxi->sample_name)) ;
 		snprintf (pxi->sample_name, sizeof (pxi->sample_name), "%s", "Sample #1") ;
@@ -252,7 +252,7 @@ dpcm_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 		bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 		while (total > 0)
 		{	len = (total > bufferlen) ? bufferlen : total ;
-			total -= dpcm_read_dles2s (psf, ubuf.sbuf, len) ;
+			total -= (int) dpcm_read_dles2s (psf, ubuf.sbuf, len) ;
 			} ;
 		}
 	else
@@ -260,7 +260,7 @@ dpcm_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 		bufferlen = ARRAY_LEN (ubuf.sbuf) ;
 		while (total > 0)
 		{	len = (total > bufferlen) ? bufferlen : total ;
-			total -= dpcm_read_dsc2s (psf, ubuf.sbuf, len) ;
+			total -= (int) dpcm_read_dsc2s (psf, ubuf.sbuf, len) ;
 			} ;
 		} ;
 
@@ -280,49 +280,45 @@ xi_write_header (SF_PRIVATE *psf, int UNUSED (calc_length))
 	current = psf_ftell (psf) ;
 
 	/* Reset the current header length to zero. */
-	psf->header [0] = 0 ;
-	psf->headindex = 0 ;
+	psf->header.ptr [0] = 0 ;
+	psf->header.indx = 0 ;
 	psf_fseek (psf, 0, SEEK_SET) ;
 
 	string = "Extended Instrument: " ;
-	psf_binheader_writef (psf, "b", string, strlen (string)) ;
-	psf_binheader_writef (psf, "b1", pxi->filename, sizeof (pxi->filename), 0x1A) ;
+	psf_binheader_writef (psf, "b", BHWv (string), BHWz (strlen (string))) ;
+	psf_binheader_writef (psf, "b1", BHWv (pxi->filename), BHWz (sizeof (pxi->filename)), BHW1 (0x1A)) ;
 
 	/* Write software version and two byte XI version. */
-	psf_binheader_writef (psf, "eb2", pxi->software, sizeof (pxi->software), (1 << 8) + 2) ;
+	psf_binheader_writef (psf, "eb2", BHWv (pxi->software), BHWz (sizeof (pxi->software)), BHW2 ((1 << 8) + 2)) ;
 
 	/*
 	** Jump note numbers (96), volume envelope (48), pan envelope (48),
 	** volume points (1), pan points (1)
 	*/
-	psf_binheader_writef (psf, "z", (size_t) (96 + 48 + 48 + 1 + 1)) ;
+	psf_binheader_writef (psf, "z", BHWz ((size_t) (96 + 48 + 48 + 1 + 1))) ;
 
 	/* Jump volume loop (3 bytes), pan loop (3), envelope flags (3), vibrato (3)
 	** fade out (2), 22 unknown bytes, and then write sample_count (2 bytes).
 	*/
-	psf_binheader_writef (psf, "ez2z2", (size_t) (4 * 3), 0x1234, make_size_t (22), 1) ;
+	psf_binheader_writef (psf, "ez2z2", BHWz ((size_t) (4 * 3)), BHW2 (0x1234), BHWz (22), BHW2 (1)) ;
 
 	pxi->loop_begin = 0 ;
 	pxi->loop_end = 0 ;
 
-	psf_binheader_writef (psf, "et844", psf->sf.frames, pxi->loop_begin, pxi->loop_end) ;
+	psf_binheader_writef (psf, "et844", BHW8 (psf->sf.frames), BHW4 (pxi->loop_begin), BHW4 (pxi->loop_end)) ;
 
 	/* volume, fine tune, flags, pan, note, namelen */
-	psf_binheader_writef (psf, "111111", 128, 0, pxi->sample_flags, 128, 0, strlen (pxi->sample_name)) ;
-
-	psf_binheader_writef (psf, "b", pxi->sample_name, sizeof (pxi->sample_name)) ;
-
-
-
+	psf_binheader_writef (psf, "111111", BHW1 (128), BHW1 (0), BHW1 (pxi->sample_flags), BHW1 (128), BHW1 (0), BHW1 (strlen (pxi->sample_name))) ;
 
+	psf_binheader_writef (psf, "b", BHWv (pxi->sample_name), BHWz (sizeof (pxi->sample_name))) ;
 
 	/* Header construction complete so write it out. */
-	psf_fwrite (psf->header, psf->headindex, 1, psf) ;
+	psf_fwrite (psf->header.ptr, psf->header.indx, 1, psf) ;
 
 	if (psf->error)
 		return psf->error ;
 
-	psf->dataoffset = psf->headindex ;
+	psf->dataoffset = psf->header.indx ;
 
 	if (current > 0)
 		psf_fseek (psf, current, SEEK_SET) ;
@@ -409,8 +405,6 @@ xi_read_header (SF_PRIVATE *psf)
 
 		psf_log_printf (psf, "  size    : %d\n", sample_sizes [k]) ;
 
-
-
 		psf_log_printf (psf, "  loop\n    begin : %d\n    end   : %d\n", loop_begin, loop_end) ;
 
 		psf_log_printf (psf, "  volume  : %u\n  f. tune : %d\n  flags   : 0x%02X ",
@@ -522,7 +516,7 @@ dpcm_read_dsc2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		dsc2s_array (pxi, ubuf.scbuf, readcount, ptr + total) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -548,7 +542,7 @@ dpcm_read_dsc2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		dsc2i_array (pxi, ubuf.scbuf, readcount, ptr + total) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -577,7 +571,7 @@ dpcm_read_dsc2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		dsc2f_array (pxi, ubuf.scbuf, readcount, ptr + total, normfact) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -606,7 +600,7 @@ dpcm_read_dsc2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		dsc2d_array (pxi, ubuf.scbuf, readcount, ptr + total, normfact) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -635,7 +629,7 @@ dpcm_read_dles2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		dles2s_array (pxi, ubuf.sbuf, readcount, ptr + total) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -661,7 +655,7 @@ dpcm_read_dles2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		dles2i_array (pxi, ubuf.sbuf, readcount, ptr + total) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -690,7 +684,7 @@ dpcm_read_dles2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		dles2f_array (pxi, ubuf.sbuf, readcount, ptr + total, normfact) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -719,7 +713,7 @@ dpcm_read_dles2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	while (len > 0)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
-		readcount = psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		readcount = (int) psf_fread (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		dles2d_array (pxi, ubuf.sbuf, readcount, ptr + total, normfact) ;
 		total += readcount ;
 		if (readcount < bufferlen)
@@ -760,7 +754,7 @@ dpcm_write_s2dsc (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		s2dsc_array (pxi, ptr + total, ubuf.scbuf, bufferlen) ;
-		writecount = psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -786,7 +780,7 @@ dpcm_write_i2dsc (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		i2dsc_array (pxi, ptr + total, ubuf.scbuf, bufferlen) ;
-		writecount = psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -815,7 +809,7 @@ dpcm_write_f2dsc (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		f2dsc_array (pxi, ptr + total, ubuf.scbuf, bufferlen, normfact) ;
-		writecount = psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -844,7 +838,7 @@ dpcm_write_d2dsc (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		d2dsc_array (pxi, ptr + total, ubuf.scbuf, bufferlen, normfact) ;
-		writecount = psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.scbuf, sizeof (signed char), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -871,7 +865,7 @@ dpcm_write_s2dles (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		s2dles_array (pxi, ptr + total, ubuf.sbuf, bufferlen) ;
-		writecount = psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -897,7 +891,7 @@ dpcm_write_i2dles (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		i2dles_array (pxi, ptr + total, ubuf.sbuf, bufferlen) ;
-		writecount = psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -926,7 +920,7 @@ dpcm_write_f2dles (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		f2dles_array (pxi, ptr + total, ubuf.sbuf, bufferlen, normfact) ;
-		writecount = psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -955,7 +949,7 @@ dpcm_write_d2dles (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 	{	if (len < bufferlen)
 			bufferlen = (int) len ;
 		d2dles_array (pxi, ptr + total, ubuf.sbuf, bufferlen, normfact) ;
-		writecount = psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
+		writecount = (int) psf_fwrite (ubuf.sbuf, sizeof (short), bufferlen, psf) ;
 		total += writecount ;
 		if (writecount < bufferlen)
 			break ;
@@ -1072,7 +1066,7 @@ f2dsc_array (XI_PRIVATE *pxi, const float *src, signed char *dest, int count, fl
 	last_val = pxi->last_16 >> 8 ;
 
 	for (k = 0 ; k < count ; k++)
-	{	current = lrintf (src [k] * normfact) ;
+	{	current = psf_lrintf (src [k] * normfact) ;
 		dest [k] = current - last_val ;
 		last_val = current ;
 		} ;
@@ -1088,7 +1082,7 @@ d2dsc_array (XI_PRIVATE *pxi, const double *src, signed char *dest, int count, d
 	last_val = pxi->last_16 >> 8 ;
 
 	for (k = 0 ; k < count ; k++)
-	{	current = lrint (src [k] * normfact) ;
+	{	current = psf_lrint (src [k] * normfact) ;
 		dest [k] = current - last_val ;
 		last_val = current ;
 		} ;
@@ -1202,7 +1196,7 @@ f2dles_array (XI_PRIVATE *pxi, const float *src, short *dest, int count, float n
 	last_val = pxi->last_16 ;
 
 	for (k = 0 ; k < count ; k++)
-	{	current = lrintf (src [k] * normfact) ;
+	{	current = psf_lrintf (src [k] * normfact) ;
 		diff = current - last_val ;
 		dest [k] = LE2H_16 (diff) ;
 		last_val = current ;
@@ -1219,7 +1213,7 @@ d2dles_array (XI_PRIVATE *pxi, const double *src, short *dest, int count, double
 	last_val = pxi->last_16 ;
 
 	for (k = 0 ; k < count ; k++)
-	{	current = lrint (src [k] * normfact) ;
+	{	current = psf_lrint (src [k] * normfact) ;
 		diff = current - last_val ;
 		dest [k] = LE2H_16 (diff) ;
 		last_val = current ;
@@ -1227,4 +1221,3 @@ d2dles_array (XI_PRIVATE *pxi, const double *src, short *dest, int count, double
 
 	pxi->last_16 = last_val ;
 } /* d2dles_array */
-
