@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2014 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2019 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,16 +27,20 @@
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
+#else
+#include "sf_unistd.h"
 #endif
 
 #include <math.h>
 
 #include <sndfile.h>
 
+#include "sfendian.h"
 #include "utils.h"
 
 #define	BUFFER_LEN		(1 << 10)
 #define LOG_BUFFER_SIZE	1024
+#define data_MARKER		MAKE_MARKER ('d', 'a', 't', 'a')
 
 static	void	float_norm_test			(const char *filename) ;
 static	void	double_norm_test		(const char *filename) ;
@@ -44,6 +48,8 @@ static	void	format_tests			(void) ;
 static	void	calc_peak_test			(int filetype, const char *filename, int channels) ;
 static	void	truncate_test			(const char *filename, int filetype) ;
 static	void	instrument_test			(const char *filename, int filetype) ;
+static	void	cue_test				(const char *filename, int filetype) ;
+static	void	cue_test_var			(const char *filename, int filetype, int count) ;
 static	void	channel_map_test		(const char *filename, int filetype) ;
 static	void	current_sf_info_test	(const char *filename) ;
 static	void	raw_needs_endswap_test	(const char *filename, int filetype) ;
@@ -54,7 +60,7 @@ static	void	broadcast_coding_history_test	(const char *filename) ;
 static	void	broadcast_coding_history_size	(const char *filename) ;
 
 /* Cart Chunk tests */
-static void	cart_test			(const char *filename, int filetype) ;
+static void	cart_test				(const char *filename, int filetype) ;
 static void	cart_rdwr_test			(const char *filename, int filetype) ;
 
 /* Force the start of this buffer to be double aligned. Sparc-solaris will
@@ -79,6 +85,7 @@ main (int argc, char *argv [])
 		printf ("           peak    - test peak calculation\n") ;
 		printf ("           trunc   - test file truncation\n") ;
 		printf ("           inst    - test set/get of SF_INSTRUMENT.\n") ;
+		printf ("           cue     - test set/get of SF_CUES.\n") ;
 		printf ("           chanmap - test set/get of channel map data..\n") ;
 		printf ("           bext    - test set/get of SF_BROADCAST_INFO.\n") ;
 		printf ("           bextch  - test set/get of SF_BROADCAST_INFO coding_history.\n") ;
@@ -108,8 +115,8 @@ main (int argc, char *argv [])
 	{	/*	Preliminary float/double normalisation tests. More testing
 		**	is done in the program 'floating_point_test'.
 		*/
-		float_norm_test		("float.wav") ;
-		double_norm_test	("double.wav") ;
+		float_norm_test		("cmd_float.wav") ;
+		double_norm_test	("cmd_double.wav") ;
 		test_count ++ ;
 		} ;
 
@@ -134,11 +141,27 @@ main (int argc, char *argv [])
 
 	if (do_all || strcmp (argv [1], "inst") == 0)
 	{	instrument_test ("instrument.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16) ;
-		instrument_test ("instrument.aiff" , SF_FORMAT_AIFF | SF_FORMAT_PCM_24) ;
+		/*-instrument_test ("instrument.aiff" , SF_FORMAT_AIFF | SF_FORMAT_PCM_24) ;-*/
 		/*-instrument_test ("instrument.xi", SF_FORMAT_XI | SF_FORMAT_DPCM_16) ;-*/
 		test_count ++ ;
 		} ;
 
+	if (do_all || strcmp (argv [1], "cue") == 0)
+	{	/* 2500 is close to the largest number of cues possible because of block sizes (enforced in aiff.c, wav.c) */
+		int cuecounts [] = { 0, 1, 10, 100, 101, 1000, 1001, 2500 } ;
+		unsigned int i ;
+
+		cue_test ("cue.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16) ;
+		cue_test ("cue.aiff" , SF_FORMAT_AIFF | SF_FORMAT_PCM_24) ;
+
+		for (i = 0 ; i < ARRAY_LEN (cuecounts) ; i++)
+		{	cue_test_var ("cue.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16, cuecounts [i]) ;
+			cue_test_var ("cue.aiff", SF_FORMAT_AIFF | SF_FORMAT_PCM_24, cuecounts [i]) ;
+			} ;
+
+		test_count ++ ;
+		} ;
+
 	if (do_all || strcmp (argv [1], "current_sf_info") == 0)
 	{	current_sf_info_test ("current.wav") ;
 		test_count ++ ;
@@ -216,9 +239,9 @@ float_norm_test (const char *filename)
 
 	/* Create float_data with all values being less than 1.0. */
 	for (k = 0 ; k < BUFFER_LEN / 2 ; k++)
-		float_data [k] = (k + 5) / (2.0 * BUFFER_LEN) ;
+		float_data [k] = (k + 5) / (2.0f * BUFFER_LEN) ;
 	for (k = BUFFER_LEN / 2 ; k < BUFFER_LEN ; k++)
-		float_data [k] = (k + 5) ;
+		float_data [k] = (float) (k + 5) ;
 
 	if (! (file = sf_open (filename, SFM_WRITE, &sfinfo)))
 	{	printf ("Line %d: sf_open_write failed with error : ", __LINE__) ;
@@ -229,7 +252,7 @@ float_norm_test (const char *filename)
 
 	/* Normalisation is on by default so no need to do anything here. */
 
-	if ((k = sf_write_float (file, float_data, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
+	if ((k = (unsigned int) sf_write_float (file, float_data, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
 	{	printf ("Line %d: sf_write_float failed with short write (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -237,7 +260,7 @@ float_norm_test (const char *filename)
 	/* Turn normalisation off. */
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
 
-	if ((k = sf_write_float (file, float_data + BUFFER_LEN / 2, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
+	if ((k = (unsigned int) sf_write_float (file, float_data + BUFFER_LEN / 2, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
 	{	printf ("Line %d: sf_write_float failed with short write (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -268,7 +291,7 @@ float_norm_test (const char *filename)
 		} ;
 
 	/* Read float_data and check that it is normalised (ie default). */
-	if ((k = sf_read_float (file, float_data, BUFFER_LEN)) != BUFFER_LEN)
+	if ((k = (unsigned int) sf_read_float (file, float_data, BUFFER_LEN)) != BUFFER_LEN)
 	{	printf ("\n\nLine %d: sf_read_float failed with short read (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -283,7 +306,7 @@ float_norm_test (const char *filename)
 	sf_seek (file, 0, SEEK_SET) ;
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
 
-	if ((k = sf_read_float (file, float_data, BUFFER_LEN)) != BUFFER_LEN)
+	if ((k = (unsigned int) sf_read_float (file, float_data, BUFFER_LEN)) != BUFFER_LEN)
 	{	printf ("\n\nLine %d: sf_read_float failed with short read (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -298,7 +321,7 @@ float_norm_test (const char *filename)
 	sf_seek (file, 0, SEEK_SET) ;
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_TRUE) ;
 
-	if ((k = sf_read_float (file, float_data, BUFFER_LEN)) != BUFFER_LEN)
+	if ((k = (unsigned int) sf_read_float (file, float_data, BUFFER_LEN)) != BUFFER_LEN)
 	{	printf ("\n\nLine %d: sf_read_float failed with short read (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -346,7 +369,7 @@ double_norm_test (const char *filename)
 	/* Normailsation is on by default so no need to do anything here. */
 	/*-sf_command (file, "set-norm-double", "true", 0) ;-*/
 
-	if ((k = sf_write_double (file, double_data, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
+	if ((k = (unsigned int) sf_write_double (file, double_data, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
 	{	printf ("Line %d: sf_write_double failed with short write (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -354,7 +377,7 @@ double_norm_test (const char *filename)
 	/* Turn normalisation off. */
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
 
-	if ((k = sf_write_double (file, double_data + BUFFER_LEN / 2, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
+	if ((k = (unsigned int) sf_write_double (file, double_data + BUFFER_LEN / 2, BUFFER_LEN / 2)) != BUFFER_LEN / 2)
 	{	printf ("Line %d: sf_write_double failed with short write (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -384,7 +407,7 @@ double_norm_test (const char *filename)
 		} ;
 
 	/* Read double_data and check that it is normalised (ie default). */
-	if ((k = sf_read_double (file, double_data, BUFFER_LEN)) != BUFFER_LEN)
+	if ((k = (unsigned int) sf_read_double (file, double_data, BUFFER_LEN)) != BUFFER_LEN)
 	{	printf ("\n\nLine %d: sf_read_double failed with short read (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -399,7 +422,7 @@ double_norm_test (const char *filename)
 	sf_seek (file, 0, SEEK_SET) ;
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
 
-	if ((k = sf_read_double (file, double_data, BUFFER_LEN)) != BUFFER_LEN)
+	if ((k = (unsigned int) sf_read_double (file, double_data, BUFFER_LEN)) != BUFFER_LEN)
 	{	printf ("\n\nLine %d: sf_read_double failed with short read (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -414,7 +437,7 @@ double_norm_test (const char *filename)
 	sf_seek (file, 0, SEEK_SET) ;
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_TRUE) ;
 
-	if ((k = sf_read_double (file, double_data, BUFFER_LEN)) != BUFFER_LEN)
+	if ((k = (unsigned int) sf_read_double (file, double_data, BUFFER_LEN)) != BUFFER_LEN)
 	{	printf ("\n\nLine %d: sf_read_double failed with short read (%d ->%d)\n", __LINE__, BUFFER_LEN, k) ;
 		exit (1) ;
 		} ;
@@ -509,7 +532,7 @@ format_tests	(void)
 	/* Now test subtype formats. */
 	sf_command (NULL, SFC_GET_FORMAT_SUBTYPE_COUNT, &count, sizeof (int)) ;
 
-	if (count < 0 || count > 30)
+	if (count < 0 || count > 33)
 	{	printf ("Line %d: Weird count.\n", __LINE__) ;
 		exit (1) ;
 		} ;
@@ -756,7 +779,6 @@ instrument_test (const char *filename, int filetype)
 		**	write_inst struct to hold the default value that the WAV
 		**	module should hold.
 		*/
-		write_inst.detune = 0 ;
 		write_inst.key_lo = write_inst.velocity_lo = 0 ;
 		write_inst.key_hi = write_inst.velocity_hi = 127 ;
 		write_inst.gain = 1 ;
@@ -824,6 +846,210 @@ instrument_test (const char *filename, int filetype)
 	puts ("ok") ;
 } /* instrument_test */
 
+static void
+print_cue (SF_CUES *cue, int i)
+{
+	printf ("   indx[%d]       : %d\n"
+		"   position      : %u\n"
+		"   fcc_chunk     : %x\n"
+		"   chunk_start   : %d\n"
+		"   block_start   : %d\n"
+		"   sample_offset : %u\n"
+		"   name          : %s\n",
+		i,
+		cue->cue_points [i].indx,
+		cue->cue_points [i].position,
+		cue->cue_points [i].fcc_chunk,
+		cue->cue_points [i].chunk_start,
+		cue->cue_points [i].block_start,
+		cue->cue_points [i].sample_offset,
+		cue->cue_points [i].name) ;
+}
+
+static int
+cue_compare (SF_CUES *write_cue, SF_CUES *read_cue, size_t cue_size, int line)
+{
+	if (memcmp (write_cue, read_cue, cue_size) != 0)
+	{
+		printf ("\n\nLine %d : cue comparison failed.\n\n", line) ;
+		printf ("W  Cue count     : %d\n", write_cue->cue_count) ;
+		if (write_cue->cue_count > 0)
+			print_cue (write_cue, 0) ;
+		if (write_cue->cue_count > 2)	/* print last if at least 2 */
+			print_cue (write_cue, write_cue->cue_count - 1) ;
+
+		printf ("R  Cue count     : %d\n", read_cue->cue_count) ;
+		if (read_cue->cue_count > 0)
+			print_cue (read_cue, 0) ;
+		if (read_cue->cue_count > 2)	/* print last if at least 2 */
+			print_cue (read_cue, read_cue->cue_count - 1) ;
+
+		return SF_FALSE ;
+		} ;
+
+	return SF_TRUE ;
+} /* cue_compare */
+
+static void
+cue_rw_test (const char *filename)
+{	SNDFILE *sndfile ;
+	SF_INFO sfinfo ;
+	SF_CUES cues ;
+	memset (&sfinfo, 0, sizeof (SF_INFO)) ;
+
+	sndfile = test_open_file_or_die (filename, SFM_RDWR, &sfinfo, SF_FALSE, __LINE__) ;
+
+	exit_if_true (
+		sf_command (sndfile, SFC_GET_CUE_COUNT, &cues.cue_count, sizeof (cues.cue_count)) != SF_TRUE,
+		"\nLine %d: SFC_GET_CUE_COUNT command failed.\n\n", __LINE__
+		) ;
+
+	exit_if_true (
+		cues.cue_count != 3,
+		"\nLine %d: Expected cue_count (%u) to be 3.\n\n", __LINE__, cues.cue_count
+		) ;
+
+	if (sf_command (sndfile, SFC_GET_CUE, &cues, sizeof (cues)) == SF_TRUE)
+	{	cues.cue_points [1].sample_offset = 3 ;
+
+		if (sf_command (sndfile, SFC_SET_CUE, &cues, sizeof (cues)) == SF_TRUE)
+			printf ("Sucess: [%s] updated\n", filename) ;
+		else
+			printf ("Error: SFC_SET_CUE on [%s] [%s]\n", filename, sf_strerror (sndfile)) ;
+		}
+	else
+		printf ("Error: SFC_GET_CUE on [%s] [%s]\n", filename, sf_strerror (sndfile)) ;
+
+
+	if (sf_command (sndfile, SFC_UPDATE_HEADER_NOW, NULL, 0) != 0)
+		printf ("Error: SFC_UPDATE_HEADER_NOW on [%s] [%s]\n", filename, sf_strerror (sndfile)) ;
+
+	sf_write_sync (sndfile) ;
+	sf_close (sndfile) ;
+
+	return ;
+} /* cue_rw_test */
+
+static void
+cue_test (const char *filename, int filetype)
+{	SF_CUES write_cue ;
+	SF_CUES read_cue ;
+	SNDFILE	*file ;
+	SF_INFO	sfinfo ;
+
+	if (filetype == (SF_FORMAT_WAV | SF_FORMAT_PCM_16))
+	{	write_cue = (SF_CUES)
+		{	2,		/* cue_count */
+			{	{	1, 0, data_MARKER, 0, 0, 1, "" },
+				{	2, 0, data_MARKER, 0, 0, 2, "" },
+			}
+		} ;
+	}
+	else
+	{	write_cue = (SF_CUES)
+		{	2,		/* cue_count */
+			{	{	1, 0, data_MARKER, 0, 0, 1, "Cue1" },
+				{	2, 0, data_MARKER, 0, 0, 2, "Cue2" },
+			}
+		} ;
+	}
+
+	print_test_name ("cue_test", filename) ;
+
+	sfinfo.samplerate	= 11025 ;
+	sfinfo.format		= filetype ;
+	sfinfo.channels		= 1 ;
+
+	file = test_open_file_or_die (filename, SFM_WRITE, &sfinfo, SF_TRUE, __LINE__) ;
+	if (sf_command (file, SFC_SET_CUE, &write_cue, sizeof (write_cue)) == SF_FALSE)
+	{	printf ("\n\nLine %d : sf_command (SFC_SET_CUE) failed.\n\n", __LINE__) ;
+		exit (1) ;
+		} ;
+	test_write_double_or_die (file, 0, double_data, BUFFER_LEN, __LINE__) ;
+	sf_close (file) ;
+
+	memset (&read_cue, 0, sizeof (read_cue)) ;
+
+	file = test_open_file_or_die (filename, SFM_READ, &sfinfo, SF_TRUE, __LINE__) ;
+	if (sf_command (file, SFC_GET_CUE, &read_cue, sizeof (read_cue)) == SF_FALSE)
+	{	printf ("\n\nLine %d : sf_command (SFC_GET_CUE) failed.\n\n", __LINE__) ;
+		exit (1) ;
+		return ;
+		} ;
+	check_log_buffer_or_die (file, __LINE__) ;
+	sf_close (file) ;
+
+	if (cue_compare (&write_cue, &read_cue, sizeof (write_cue), __LINE__) == SF_FALSE)
+			exit (1) ;
+
+	if (0) cue_rw_test (filename) ;
+
+	unlink (filename) ;
+	puts ("ok") ;
+} /* cue_test */
+
+/* calculate size of SF_CUES struct given number of cues */
+#define SF_CUES_SIZE(count)	(sizeof (uint32_t) + sizeof (SF_CUE_POINT) * (count))
+
+static void
+cue_test_var (const char *filename, int filetype, int count)
+{	size_t cues_size = SF_CUES_SIZE (count) ;
+	SF_CUES *write_cue = calloc (1, cues_size) ;
+	SF_CUES *read_cue = calloc (1, cues_size) ;
+	SNDFILE	*file ;
+	SF_INFO	sfinfo ;
+	char name [40] ;
+	int i ;
+
+	snprintf (name, sizeof (name), "cue_test_var %d", count) ;
+	print_test_name (name, filename) ;
+
+	if (write_cue == NULL || read_cue == NULL)
+	{	printf ("ok (can't alloc)\n") ;
+		return ;
+		} ;
+
+	write_cue->cue_count = count ;
+	for (i = 0 ; i < count ; i++)
+	{	write_cue->cue_points [i] = (SF_CUE_POINT) { i, 0, data_MARKER, 0, 0, i, "" } ;
+		if (filetype == (SF_FORMAT_AIFF | SF_FORMAT_PCM_24))
+			snprintf (write_cue->cue_points [i].name, sizeof (write_cue->cue_points [i].name), "Cue%03d", i) ;
+		} ;
+
+	sfinfo.samplerate	= 11025 ;
+	sfinfo.format		= filetype ;
+	sfinfo.channels		= 1 ;
+
+	file = test_open_file_or_die (filename, SFM_WRITE, &sfinfo, SF_TRUE, __LINE__) ;
+	if (sf_command (file, SFC_SET_CUE, write_cue, (int) cues_size) == SF_FALSE)
+	{	printf ("\n\nLine %d : sf_command (SFC_SET_CUE) failed with %d cues, datasize %zu --> error: %s\n\n", __LINE__, count, cues_size, sf_strerror (file)) ;
+		exit (1) ;
+		} ;
+	test_write_double_or_die (file, 0, double_data, BUFFER_LEN, __LINE__) ;
+	sf_close (file) ;
+
+	memset (read_cue, 0, cues_size) ;
+
+	file = test_open_file_or_die (filename, SFM_READ, &sfinfo, SF_TRUE, __LINE__) ;
+
+	if (sf_command (file, SFC_GET_CUE, read_cue, (int) cues_size) == SF_FALSE)
+	{	printf ("\n\nLine %d : sf_command (SFC_GET_CUE) failed with %d cues, datasize %zu --> error: %s\n\n", __LINE__, count, cues_size, sf_strerror (file)) ;
+		exit (1) ;
+		} ;
+	check_log_buffer_or_die (file, __LINE__) ;
+	sf_close (file) ;
+
+	if (cue_compare (write_cue, read_cue, cues_size, __LINE__) == SF_FALSE)
+	{	printf ("\n\nLine %d : cue_compare failed.\n\n", __LINE__) ;
+		exit (1) ;
+		} ;
+
+	free (write_cue) ;
+	free (read_cue) ;
+	unlink (filename) ;
+	puts ("ok") ;
+} /* cue_test_var */
+
 static	void
 current_sf_info_test	(const char *filename)
 {	SNDFILE *outfile, *infile ;
@@ -880,6 +1106,7 @@ broadcast_test (const char *filename, int filetype)
 
 	print_test_name ("broadcast_test", filename) ;
 
+	memset (&sfinfo, 0, sizeof (sfinfo)) ;
 	sfinfo.samplerate	= 11025 ;
 	sfinfo.format		= filetype ;
 	sfinfo.channels		= 1 ;
@@ -913,7 +1140,7 @@ broadcast_test (const char *filename, int filetype)
 	check_log_buffer_or_die (file, __LINE__) ;
 	sf_close (file) ;
 
-	if (bc_read.version != 1)
+	if (bc_read.version != 2)
 	{	printf ("\n\nLine %d : Read bad version number %d.\n\n", __LINE__, bc_read.version) ;
 		exit (1) ;
 		return ;
@@ -1124,7 +1351,7 @@ broadcast_coding_history_test (const char *filename)
 		exit (1) ;
 		} ;
 
-	bc_write.coding_history_size = strlen (supplied_history) ;
+	bc_write.coding_history_size = (uint32_t) strlen (supplied_history) ;
 	bc_write.coding_history_size = snprintf (bc_write.coding_history, sizeof (bc_write.coding_history), "%s", supplied_history) ;
 
 	file = test_open_file_or_die (filename, SFM_WRITE, &sfinfo, SF_TRUE, __LINE__) ;
@@ -1196,7 +1423,7 @@ broadcast_coding_history_size (const char *filename)
 	for (k = 0 ; bc_write.coding_history_size < 512 ; k++)
 	{	snprintf (bc_write.coding_history + bc_write.coding_history_size,
 			sizeof (bc_write.coding_history) - bc_write.coding_history_size, "line %4d\n", k) ;
-		bc_write.coding_history_size = strlen (bc_write.coding_history) ;
+		bc_write.coding_history_size = (uint32_t) strlen (bc_write.coding_history) ;
 		} ;
 
 	exit_if_true (bc_write.coding_history_size < 512,
@@ -1242,6 +1469,7 @@ cart_test (const char *filename, int filetype)
 
 	print_test_name ("cart_test", filename) ;
 
+	memset (&sfinfo, 0, sizeof (sfinfo)) ;
 	sfinfo.samplerate	= 11025 ;
 	sfinfo.format		= filetype ;
 	sfinfo.channels		= 1 ;
@@ -1265,7 +1493,7 @@ cart_test (const char *filename, int filetype)
 	ca_write.level_reference = 42 ;
 	snprintf (ca_write.url, sizeof (ca_write.url), "http://www.test.com/test_url") ;
 	snprintf (ca_write.tag_text, sizeof (ca_write.tag_text), "tag text test! \r\n") ; // must be terminated \r\n to be valid
-	ca_write.tag_text_size = strlen (ca_write.tag_text) ;
+	ca_write.tag_text_size = (uint32_t) strlen (ca_write.tag_text) ;
 
 	file = test_open_file_or_die (filename, SFM_WRITE, &sfinfo, SF_TRUE, __LINE__) ;
 	if (sf_command (file, SFC_SET_CART_INFO, &ca_write, sizeof (ca_write)) == SF_FALSE)
@@ -1424,7 +1652,7 @@ cart_rdwr_test (const char *filename, int filetype)
 	cinfo.level_reference = 42 ;
 	snprintf (cinfo.url, sizeof (cinfo.url), "http://www.test.com/test_url") ;
 	snprintf (cinfo.tag_text, sizeof (cinfo.tag_text), "tag text test!\r\n") ;
-	cinfo.tag_text_size = strlen (cinfo.tag_text) ;
+	cinfo.tag_text_size = (uint32_t) strlen (cinfo.tag_text) ;
 
 	file = test_open_file_or_die (filename, SFM_RDWR, &sfinfo, SF_TRUE, __LINE__) ;
 	frames = sfinfo.frames ;
