@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2011 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2012 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,13 +21,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <math.h>
+#include <inttypes.h>
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
+#else
+#include "sf_unistd.h"
 #endif
 
-#include <math.h>
-
 #include <sndfile.h>
 
 #include "utils.h"
@@ -175,7 +177,7 @@ flt_scale_clip_test_16 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -305,7 +307,7 @@ flt_scale_clip_test_24 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -435,7 +437,7 @@ flt_scale_clip_test_32 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -565,7 +567,7 @@ flt_scale_clip_test_08 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -697,7 +699,7 @@ dbl_scale_clip_test_16 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -827,7 +829,7 @@ dbl_scale_clip_test_24 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -957,7 +959,7 @@ dbl_scale_clip_test_32 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1087,7 +1089,7 @@ dbl_scale_clip_test_08 (const char *filename, int filetype, float maxval)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1219,7 +1221,7 @@ static void flt_short_clip_read_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1292,7 +1294,7 @@ static void flt_int_clip_read_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1366,7 +1368,7 @@ static void dbl_short_clip_read_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1439,7 +1441,7 @@ static void dbl_int_clip_read_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1516,7 +1518,7 @@ static void short_flt_scale_write_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != 3 * BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, 3 * BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, 3 * BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1609,7 +1611,7 @@ static void short_dbl_scale_write_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != 3 * BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, 3 * BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, 3 * BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1703,7 +1705,7 @@ static void int_flt_scale_write_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != 3 * BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, 3 * BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, 3 * BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
@@ -1796,7 +1798,7 @@ static void int_dbl_scale_write_test (const char *filename, int filetype)
 		} ;
 
 	if (sfinfo.frames != 3 * BUFFER_SIZE)
-	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %ld).\n\n", __LINE__, 3 * BUFFER_SIZE, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("\n\nLine %d: Incorrect number of frames in file (%d => %" PRId64 ").\n\n", __LINE__, 3 * BUFFER_SIZE, sfinfo.frames) ;
 		exit (1) ;
 		} ;
 
